@@ -17,7 +17,7 @@
 #![deny(dead_code, missing_docs, warnings)]
 
 //! Derivation macros for strict encoding. To learn more about the strict
-//! encoding please check [`::strict_encoding`] crate.
+//! encoding please check `strict_encoding` crate.
 //!
 //! # Derivation macros
 //!
@@ -172,35 +172,76 @@
 
 extern crate proc_macro;
 #[macro_use]
-extern crate amplify;
-#[macro_use]
-extern crate quote;
-#[macro_use]
 extern crate syn;
+#[macro_use]
+extern crate amplify_syn;
 
-mod decode;
-mod encode;
-mod param;
-
+use encoding_derive_helpers::{decode_derive, encode_derive, TlvEncoding};
 use proc_macro::TokenStream;
 use syn::DeriveInput;
 
-pub(crate) const ATTR_NAME: &str = "strict_encoding";
-
-/// Derives [`::strict_encoding::StrictEncode`] implementation for the type.
+/// Derives [`StrictEncode`] implementation for the type.
 #[proc_macro_derive(StrictEncode, attributes(strict_encoding))]
 pub fn derive_strict_encode(input: TokenStream) -> TokenStream {
     let derive_input = parse_macro_input!(input as DeriveInput);
-    encode::encode_derive(derive_input)
-        .unwrap_or_else(|e| e.to_compile_error())
-        .into()
+    encode_derive(
+        "strict_encoding",
+        ident!(StrictEncode),
+        ident!(strict_encode),
+        ident!(strict_serialize),
+        derive_input,
+        TlvEncoding::Denied,
+    )
+    .unwrap_or_else(|e| e.to_compile_error())
+    .into()
 }
 
-/// Derives [`::strict_encoding::StrictDncode`] implementation for the type.
+/// Derives [`StrictDecode`] implementation for the type.
 #[proc_macro_derive(StrictDecode, attributes(strict_encoding))]
 pub fn derive_strict_decode(input: TokenStream) -> TokenStream {
     let derive_input = parse_macro_input!(input as DeriveInput);
-    decode::decode_derive(derive_input)
-        .unwrap_or_else(|e| e.to_compile_error())
-        .into()
+    decode_derive(
+        "strict_encoding",
+        ident!(StrictDecode),
+        ident!(strict_decode),
+        ident!(strict_deserialize),
+        derive_input,
+        TlvEncoding::Denied,
+    )
+    .unwrap_or_else(|e| e.to_compile_error())
+    .into()
+}
+
+/// Derives [`StrictEncode`] implementation for the type, also providing TLV
+/// extension support.
+#[proc_macro_derive(NetworkEncode, attributes(network_encoding))]
+pub fn derive_network_encode(input: TokenStream) -> TokenStream {
+    let derive_input = parse_macro_input!(input as DeriveInput);
+    encode_derive(
+        "network_encoding",
+        ident!(StrictEncode),
+        ident!(strict_encode),
+        ident!(strict_serialize),
+        derive_input,
+        TlvEncoding::Count,
+    )
+    .unwrap_or_else(|e| e.to_compile_error())
+    .into()
+}
+
+/// Derives [`StrictDecode`] implementation for the type, also providing TLV
+/// extension support.
+#[proc_macro_derive(NetworkDecode, attributes(network_encoding))]
+pub fn derive_network_decode(input: TokenStream) -> TokenStream {
+    let derive_input = parse_macro_input!(input as DeriveInput);
+    decode_derive(
+        "network_encoding",
+        ident!(StrictDecode),
+        ident!(strict_decode),
+        ident!(strict_deserialize),
+        derive_input,
+        TlvEncoding::Count,
+    )
+    .unwrap_or_else(|e| e.to_compile_error())
+    .into()
 }
